@@ -157,10 +157,10 @@ json_array_unref (JsonArray *array)
  *
  * Gets the elements of a #JsonArray as a list of #JsonNode<!-- -->s.
  *
- * Return value: a #GList containing the elements of the array. The
- *   contents of the list are owned by the array and should never be
- *   modified or freed. Use g_list_free() on the returned list when
- *   done using it
+ * Return value: (element-type JsonNode) (transfer container): a #GList
+ *   containing the elements of the array. The contents of the list are
+ *   owned by the array and should never be modified or freed. Use
+ *   g_list_free() on the returned list when done using it
  */
 GList *
 json_array_get_elements (JsonArray *array)
@@ -186,8 +186,8 @@ json_array_get_elements (JsonArray *array)
  * Retrieves a copy of the #JsonNode containing the value of the
  * element at @index_ inside a #JsonArray
  *
- * Return value: a copy of the #JsonNode at the requested index.
- *   Use json_node_free() when done.
+ * Return value: (transfer full): a copy of the #JsonNode at the requested
+ *   index. Use json_node_free() when done.
  *
  * Since: 0.6
  */
@@ -343,7 +343,10 @@ json_array_get_string_element (JsonArray *array,
 
   node = g_ptr_array_index (array->elements, index_);
   g_return_val_if_fail (node != NULL, NULL);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_VALUE, NULL);
+  g_return_val_if_fail (JSON_NODE_HOLDS_VALUE (node) || JSON_NODE_HOLDS_NULL (node), NULL);
+
+  if (JSON_NODE_HOLDS_NULL (node))
+    return NULL;
 
   return json_node_get_string (node);
 }
@@ -386,7 +389,7 @@ json_array_get_null_element (JsonArray *array,
  *
  * See also: json_array_get_element(), json_node_get_array()
  *
- * Return value: the array
+ * Return value: (transfer none): the array
  *
  * Since: 0.8
  */
@@ -401,7 +404,10 @@ json_array_get_array_element (JsonArray *array,
 
   node = g_ptr_array_index (array->elements, index_);
   g_return_val_if_fail (node != NULL, NULL);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_ARRAY, NULL);
+  g_return_val_if_fail (JSON_NODE_HOLDS_ARRAY (node) || JSON_NODE_HOLDS_NULL (node), NULL);
+
+  if (JSON_NODE_HOLDS_NULL (node))
+    return NULL;
 
   return json_node_get_array (node);
 }
@@ -416,7 +422,7 @@ json_array_get_array_element (JsonArray *array,
  *
  * See also: json_array_get_element(), json_node_get_object()
  *
- * Return value: the object
+ * Return value: (transfer none): the object
  *
  * Since: 0.8
  */
@@ -431,7 +437,10 @@ json_array_get_object_element (JsonArray *array,
 
   node = g_ptr_array_index (array->elements, index_);
   g_return_val_if_fail (node != NULL, NULL);
-  g_return_val_if_fail (JSON_NODE_TYPE (node) == JSON_NODE_OBJECT, NULL);
+  g_return_val_if_fail (JSON_NODE_HOLDS_OBJECT (node) || JSON_NODE_HOLDS_NULL (node), NULL);
+
+  if (JSON_NODE_HOLDS_NULL (node))
+    return NULL;
 
   return json_node_get_object (node);
 }
@@ -565,8 +574,13 @@ json_array_add_string_element (JsonArray   *array,
   g_return_if_fail (array != NULL);
   g_return_if_fail (value != NULL);
 
-  node = json_node_new (JSON_NODE_VALUE);
-  json_node_set_string (node, value);
+  if (value != NULL)
+    {
+      node = json_node_new (JSON_NODE_VALUE);
+      json_node_set_string (node, value);
+    }
+  else
+    node = json_node_new (JSON_NODE_NULL);
 
   g_ptr_array_add (array->elements, node);
 }
@@ -614,8 +628,13 @@ json_array_add_array_element (JsonArray *array,
   g_return_if_fail (array != NULL);
   g_return_if_fail (value != NULL);
 
-  node = json_node_new (JSON_NODE_ARRAY);
-  json_node_take_array (node, value);
+  if (value != NULL)
+    {
+      node = json_node_new (JSON_NODE_ARRAY);
+      json_node_take_array (node, value);
+    }
+  else
+    node = json_node_new (JSON_NODE_NULL);
 
   g_ptr_array_add (array->elements, node);
 }
@@ -641,8 +660,13 @@ json_array_add_object_element (JsonArray  *array,
   g_return_if_fail (array != NULL);
   g_return_if_fail (value != NULL);
 
-  node = json_node_new (JSON_NODE_OBJECT);
-  json_node_take_object (node, value);
+  if (value != NULL)
+    {
+      node = json_node_new (JSON_NODE_OBJECT);
+      json_node_take_object (node, value);
+    }
+  else
+    node = json_node_new (JSON_NODE_NULL);
 
   g_ptr_array_add (array->elements, node);
 }
@@ -669,7 +693,7 @@ json_array_remove_element (JsonArray *array,
  * json_array_foreach_element:
  * @array: a #JsonArray
  * @func: the function to be called on each element
- * @data: data to be passed to the function
+ * @data: (allow-none): data to be passed to the function
  *
  * Iterates over all elements of @array and calls @func on
  * each one of them.
